
mvn_sim <- function(K,n,nt,delta,s) {
   Y <- as.matrix(rbinom(n+nt,1,0.5))
   X <- matrix(ncol=K,nrow=(n+nt),rnorm((n+nt)*K,0,s))
   X[,2] <- 1
   X[,1] <- X[,1]+(2*Y-1)*delta
   X[,3] <- X[,3]+(2*Y-1)*delta
   list(Y=Y[1:n],X=X[1:n,],Yt=Y[(n+1):(n+nt)],Xt=X[(n+1):(n+nt),])
}


fivedot_sim <- function(K,n,nt) {
   X <- matrix(ncol=K,nrow=(n+nt),runif((n+nt)*K,-1,1))
   X[,2] <- 1
   
   zero <- rbinom(n+nt,1,0.25)  
   X[,1] <- as.matrix((2*rbinom(n+nt,1,0.5)-1)*zero)
   X[,3] <- as.matrix((2*rbinom(n+nt,1,0.5)-1)*zero)
   Y <- as.matrix(1-zero)
   list(Y=Y[1:n],X=X[1:n,],Yt=Y[(n+1):(n+nt)],Xt=X[(n+1):(n+nt),])
}


V_g <- function(x_g, const) {
  const*solve(t(x_g)%*%x_g+0.000001*diag(ncol(x_g)))
}


P_beta <- function(Y, X, Beta, lambda, psi, const) {
   n <- nrow(X)
   m <- ncol(X)
   Gamma<-as.matrix(as.numeric(Beta!=0))

   ####size <- Gamma[1]+1+sum(Gamma[3:m])
   size <- sum(Gamma)   

   ####Any problem here?####
   prglog <- (size-1)*log(lambda)+(m-size)*log(1-lambda)
   x_g <- X[,Gamma==1]
   v_g <- V_g(x_g,const)
   b_g <- Beta[Gamma==1]
   prblog <- -0.5*t(b_g)%*%solve(v_g)%*%b_g-0.5*log(det(v_g)); 
 
   err <- rep(0,n)
   for(i in 1:n){
     if (Y[i]>0.5 & X[i,]%*%Beta<=0.0) err[i]=1
     if (Y[i]<=0.5 & X[i,]%*%Beta>0.0) err[i]=1
   }

   -n*psi*mean(err)+prglog+prblog
}
 

Beta_update <- function(Y, X, Beta, lambda, m_size, 
                        psi, pd, sigmadd, sigmaq, const) {

  Gamma<-as.matrix(as.numeric(Beta!=0))
  lg <- length(Gamma)
  l1 <- sum(Gamma!=0) 

  p_oldlog <- P_beta(Y,X,Beta,lambda,psi,const)

  Beta_new <- Beta
  
  ###Add or delete###
  if(runif(1) < pd){
    id <- sample(3:lg,1)
    Gamma[id] <- ifelse(Gamma[id],0,1)     
    Beta_new[id] <- ifelse(Gamma[id]==0,0,rnorm(1,0,sigmadd))
    p_oldlog2 <- ifelse(Gamma[id]==0,
                        dnorm(Beta[id],0,sigmadd,log=TRUE),0)           
    p_newlog2 <- ifelse(Gamma[id]==0,0,
                        dnorm(Beta_new[id],0,sigmadd,log=TRUE))            
  }
  ###Swap###
  else {
    if(l1>2){
      id1 <- ifelse(l1==3, seq(3,lg)[Gamma[-c(1,2)]==1], 
                  sample(seq(3,lg)[Gamma[-c(1,2)]==1],1))
      id2 <- sample(seq(3,lg)[Gamma[-c(1,2)]==0],1) 
      Gamma[id1] <- 0
      Gamma[id2] <- 1
      Beta_new[id1] <- 0
      Beta_new[id2] <- rnorm(1,0,sigmadd)   
      p_oldlog2 <- dnorm(Beta[id1],0,sigmadd,log=TRUE)     
      p_newlog2 <- dnorm(Beta_new[id2],0,sigmadd,log=TRUE)                                 
    }
    else {
      p_oldlog2 <- 0
      p_newlog2 <- 0
    }
  }

  ###Updata I,II###
  if(sum(Gamma!=0)<= (m_size+1)){ 
    p_newlog <- P_beta(Y,X,Beta_new,lambda,psi,const)  
    alp <- exp(min(0,p_newlog-p_oldlog+p_oldlog2-p_newlog2))  
    if(runif(1)<alp) {
      Beta <- Beta_new
      p_oldlog <- p_newlog
    }
  }
  
  ###Update III###  
  Beta_new <- Beta
  Beta_new[Beta_new!=0] <- Beta_new[Beta_new!=0] + 
                           rnorm(sum(Beta_new!=0),0,sigmaq)
  p_newlog <- P_beta(Y,X,Beta_new,lambda,psi,const)
  alp <- exp(min(0,p_newlog-p_oldlog))  
  if(runif(1)<alp) Beta <- Beta_new

  Beta      
}




###Main function for PMH method###
gp_pmh <- function(Y, X, beta_0=NULL, N=500, burn=1500, lambda=0.05, 
                   m_size=4, psi=1, pd=0.5, sigmadd=1, sigmaq=1, const=nrow(X)) {

  n <- nrow(X)
  m <- ncol(X)
  
  ###Initial values###
  Beta_mc <- matrix(ncol=N,nrow=m)  
  if(is.null(beta_0)) Beta_mc[,1] <- as.matrix(c(1,1,rep(0,m-2)))
  else Beta_mc[,1] <- as.matrix(beta_0)  

  ###Burn in###
  bi <- 1
  while(bi < burn) {
    Beta_mc[,1] <- Beta_update(Y, X, Beta_mc[,1], lambda, 
                               m_size, psi, pd, sigmadd,sigmaq,const)
    bi <- bi+1
  }
  
  ###Last N steps###
  for(k in 1:(N-1)) {
    Beta_mc[,k+1] <- Beta_update(Y, X, Beta_mc[,k], lambda, 
                                 m_size, psi, pd, sigmadd,sigmaq,const)
  }

  Beta_mc
}


















########################M-H sampler for Gamma#############

###Calculate X_gamma###
X_g <- function(X, Gamma) as.matrix(X[,Gamma==1][,-1])


###Calculate p(beta_1|Gamma,Z)###

p_b1 <- function(X, b1, Gamma, Z, sigma, log=FALSE, const) {
  
  n <- nrow(X)
  #l1 <- sum(Gamma!=0) 
  #if(l1<=1) 0.5
  #else {
    x_g <- X_g(X,Gamma)
    v_g <- V_g(x_g,const)
    Z_b1 <- Z-X[,1]*b1
    
    if(log==TRUE) {
        0.5/(sigma^2)*t(Z_b1)%*%
	(x_g%*%solve(sigma^2*solve(v_g)+
	t(x_g)%*%x_g)%*%t(x_g)-diag(n))%*%Z_b1
    }
    else {
        exp(0.5/(sigma^2)*t(Z_b1)%*%
	(x_g%*%solve(sigma^2*solve(v_g)+
	t(x_g)%*%x_g)%*%t(x_g)-diag(n))%*%Z_b1)
    }

  #}
}


###Calculate p(gamma|beta_1,Z)
p_g <- function(X, Gamma, p_b1, lambda, sigma, log=FALSE, const) {  

  n <- nrow(X)
  l1 <- sum(Gamma!=0) 
  #if(l1<=1) 0
  #else {  
    lg <- length(Gamma)
    x_g <- X_g(X,Gamma)
    v_g <- V_g(x_g,const)

    if(log==TRUE) {
      (l1-1)*log(lambda)+(lg-l1)*log(1-lambda)+
      p_b1-0.5*log(det(diag(l1-1)+sigma^(-2)*t(x_g)%*%x_g%*%v_g))
    }
    else {
      lambda^(l1-1)*(1-lambda)^(lg-l1)*
      p_b1*(det(diag(l1-1)+sigma^(-2)*t(x_g)%*%x_g%*%v_g))^(-0.5)
    }

  #}
}


###Propose new Gamma#####
###Always include x1 and x2####

Gamma_prop <- function(Gamma, pd) {
  lg <- length(Gamma)
  l1 <- sum(Gamma!=0) 
  U <- runif(1)
  
    if(U < pd) {
      id <- sample(3:lg,1)
      Gamma[id] <- ifelse(Gamma[id],0,1)
    }
    else {
      if(l1>2){
        id1 <- ifelse(l1==3, seq(3,lg)[Gamma[-c(1,2)]==1], 
                  sample(seq(3,lg)[Gamma[-c(1,2)]==1],1))
        id2 <- sample(seq(3,lg)[Gamma[-c(1,2)]==0],1) 
        Gamma[id1] <- 0
        Gamma[id2] <- 1
      }
    }

  Gamma
}





###Metropolis sampler for Gamma###
Gamma_mh <- function (X, Gamma, Z, pd,lambda, sigma ,m_size , log=FALSE, const) { 
   
  p_pos <- p_b1(X, 1,Gamma,Z,sigma,log=log, const)
  p_neg <- p_b1(X, -1,Gamma,Z,sigma,log=log, const)
  
  if(p_pos==p_neg) pp <- 0.5  
  else pp <- ifelse(log==FALSE,p_pos/(p_neg+p_pos),1/(1+exp(p_neg-p_pos)))  
 
  beta_1 <- ifelse(runif(1)<pp,1,-1)  

  Gamma_new <- Gamma_prop(Gamma, pd)  

  if(sum(Gamma_new!=0)<=m_size+1) {

    pg_old <- p_g(X, Gamma, 
                ifelse(beta_1==1,p_pos,p_neg),lambda,sigma,log=log,const)  
 
    pg_new <- p_g(X, Gamma_new, 
                p_b1(X, beta_1,Gamma_new,Z,sigma,log=log,const),lambda,sigma,log=log,const)    

    if(log==FALSE){
      if(runif(1)<min(ifelse(pg_old==0,1,pg_new/pg_old),1)) Gamma <- Gamma_new
    }
    else{
      if(runif(1)<min(ifelse(pg_old==-Inf,1,exp(pg_new-pg_old)),1)) Gamma <- Gamma_new
    }
  }

  list(Gamma=Gamma,beta_1=beta_1)
}




########################Sampler for Beta#############################
      
Beta_gb <- function (X, beta_1, Z, Gamma, sigma, const) {

  n <- nrow(X)
  m <- ncol(X)
  Beta <- matrix(ncol=1,nrow=m,0)
  Beta[1] <- beta_1

  #if (sum(Gamma!=0)<=1) Beta[2:m] <- 0
  #else {    
    x_g <- X_g(X,Gamma)
    v_g <- V_g(x_g,const)
    Z_b1 <- Z-X[,1]*beta_1  
    part1 <- solve(sigma^2*solve(v_g)+t(x_g)%*%x_g)	  
    mean <- part1%*%t(x_g)%*%Z_b1
    Sigma <- sigma^2*part1

    Beta[seq(2,m)[Gamma[-1]==1]] <- tryCatch(rmvnorm(1,mean,Sigma),error=function(e)mean)
  #}

  Beta
}  


########################Gibbs Sampler for Z################

###Calculate a+ and a-###
a_pm <- function(Y, psi) {

  p01 <- c(1,exp(psi))/(1+exp(psi)) 

  a0 <- p01[1]^Y*(1-p01[1])^(1-Y)
  a1 <- p01[2]^Y*(1-p01[2])^(1-Y)

  ap <- a1/apply(cbind(a0,a1),1,max)
  am <- a0/apply(cbind(a0,a1),1,max)

  list(ap=ap,am=am)
}


###Sampler for Z###
Z_gb <- function (X, Z, Beta, Gamma, ap ,am ,sigma) {
  
  n <- nrow(X)
  m <- ncol(X)

  mu <- X%*%(Beta*Gamma)
  Z_s <- rnorm(n,mu,sigma) 

  U <- runif(n,0,1)
  c1 <- seq(1,n)[(Z_s>0)*(U<=ap)==1]
  c2 <- seq(1,n)[(Z_s<=0)*(U<=am)==1]

  Z[c1] <- Z_s[c1]
  Z[c2] <- Z_s[c2]  

  Z
}



########################Main Function for MWG##############

gp_mwg <- function(Y, X, Z_0=NULL, Gamma_0=NULL, N=500,burn=1500, 
                   lambda=0.05, m_size=4, psi=1, pd=0.5, sigma=0.2, const=nrow(X), log=FALSE) {
  
  require(mvtnorm) 
  n <- nrow(X)
  m <- ncol(X)
  apm <- a_pm(Y,psi)
  
  Beta_mc <- matrix(ncol=N,nrow=m)
  
  if(is.null(Gamma_0)) Gamma <- as.matrix(c(1,1,rep(0,m-2)))
  else Gamma <- Gamma_0
  if(is.null(Z_0)) Z <- rep(0,n)
  else Z <- Z_0

  bi <- 1
  while(bi < burn) {
     Gamma_mc <- Gamma_mh(X, Gamma,Z,pd,lambda,sigma,m_size,log,const) 
       Gamma <- Gamma_mc$Gamma
       beta_1 <- Gamma_mc$beta_1 
     Beta_mc[,1] <- Beta_gb(X, beta_1, Z, Gamma,sigma,const)
     Z <- Z_gb(X, Z, Beta_mc[,1], Gamma,apm$ap,apm$am,sigma) 
     bi <- bi + 1
  }  
 
  for(k in 1:(N-1)) {

     Gamma_mc <- Gamma_mh(X, Gamma, Z,pd,lambda,sigma,m_size,log,const) 
       Gamma <- Gamma_mc$Gamma
       beta_1 <- Gamma_mc$beta_1     
     Beta_mc[,k+1] <- Beta_gb(X, beta_1,Z, Gamma,sigma,const)
     Z <- Z_gb(X, Z, Beta_mc[,k+1], Gamma,apm$ap,apm$am,sigma) 
  }

  Beta_mc
}











p_g_gb <- function(X, beta_1,id,Gamma,Z,lambda,sigma,log,const) {
  
  Gamma[id] <- 0 
    x_g <- X_g(X,Gamma)
    v_g <- V_g(x_g,const) 
    if(log==FALSE) a <- (1-lambda)*p_b1(X, beta_1,Gamma,Z,sigma,log,const)*
      (det(diag(sum(Gamma)-1)+sigma^(-2)*t(x_g)%*%x_g%*%v_g))^(-0.5)
    else a <- log(1-lambda)+p_b1(X, beta_1,Gamma,Z,sigma,log,const)-
         0.5*log(det(diag(sum(Gamma)-1)+sigma^(-2)*t(x_g)%*%x_g%*%v_g)) 
  Gamma[id] <- 1
    x_g <- X_g(X,Gamma)
    v_g <- V_g(x_g,const) 
    if(log==FALSE) b <- (1-lambda)*p_b1(X, beta_1,Gamma,Z,sigma,log,const)*
      (det(diag(sum(Gamma)-1)+sigma^(-2)*t(x_g)%*%x_g%*%v_g))^(-0.5)
    else b <- log(1-lambda)+p_b1(X, beta_1,Gamma,Z,sigma,log,const)-
         0.5*log(det(diag(sum(Gamma)-1)+sigma^(-2)*t(x_g)%*%x_g%*%v_g)) 
  
  ifelse(log==FALSE,b/(a+b), 1/(1+exp(a-b)))

}





Gamma_gb <- function (X, Gamma,Z,lambda,sigma,m_size,log,const) {  

  n <- nrow(X)
  m <- ncol(X)
  
  p_pos <- p_b1(X,1,Gamma,Z,sigma,log,const)
  p_neg <- p_b1(X,-1,Gamma,Z,sigma,log,const)
  
  if(p_pos==p_neg) pp <- 0.5  
  else pp <- ifelse(log==FALSE,p_pos/(p_neg+p_pos),1/(1+exp(p_neg-p_pos)))  

  beta_1 <- 2*rbinom(1,1,pp)-1  

  #print(paste("beta_1",beta_1))
  seq <- sample(3:m,m-2,replace=FALSE)    
  
  for(i in seq) { 
     p_g <- p_g_gb(X,beta_1,i,Gamma,Z,lambda,sigma,log,const)
     if(sum(Gamma)<=m_size+1) Gamma[i] <- rbinom(1,1,p_g)
  }

  list(Gamma=Gamma,beta_1=beta_1)
}


###Main function for section 7 algorithm###
gp_gibbs <- function(Y, X, Z_0=NULL, Gamma_0=NULL, N=500,burn=1500, 
                     lambda=0.05, m_size=4, psi=1, sigma=0.2, const=nrow(X), log=FALSE) {
  
  require(mvtnorm)
  n <- nrow(X)
  m <- ncol(X)
  apm <- a_pm(Y,psi)
  
  Beta_mc <- matrix(ncol=N,nrow=m)

  if(is.null(Gamma_0)) Gamma <- as.matrix(c(1,1,rep(0,m-2)))
  else Gamma <- Gamma_0
  if(is.null(Z_0)) Z <- rep(0,n)
  else Z <- Z_0
 
  bi <- 1
  while(bi < burn) {
     Gamma_mc <- Gamma_gb(X,Gamma,Z,lambda,sigma,m_size,log,const) 
       Gamma <- Gamma_mc$Gamma
       beta_1 <- Gamma_mc$beta_1    
     Beta_mc[,1] <- Beta_gb(X,beta_1,Z,Gamma,sigma,const)
     Z <- Z_gb(X, Z, Beta_mc[,1], Gamma,apm$ap,apm$am,sigma) 
     bi <- bi + 1
  }  
 
  for(k in 1:(N-1)) {

     Gamma_mc <- Gamma_gb(X,Gamma,Z,lambda,sigma,m_size,log,const) 
       Gamma <- Gamma_mc$Gamma
       beta_1 <- Gamma_mc$beta_1     
     Beta_mc[,k+1] <- Beta_gb(X,beta_1,Z,Gamma,sigma,const)
     Z <- Z_gb(X, Z, Beta_mc[,k+1], Gamma,apm$ap,apm$am,sigma) 
  #print(paste("k=",k))
  }

  Beta_mc
}





