\name{Simulate data}
\alias{fivedot_sim}
\alias{mvn_sim}
\title{Simulate Binary Classification Data} 
\description{
  Simulate data from mvn model and fivedot model described in Chen, Jiang and Tanner (2009).
}

\usage{
mvn_sim(K,n,nt,delta,s)
fivedot_sim(K,n,nt)
}
\arguments{
  \item{K}{number of predictors.}
  \item{n}{sample size of training dataset.}
  \item{nt}{sample size of test dataset}
  \item{delta,s}{mvn model parameters. See details in Chen, Jiang and Tanner 2009.}
}

\details{
  See details in Chen, Jiang and Tanner (2009). 
}

\value{
  \item{Y}{response vector for training dataset.}
  \item{X}{covariates matrix for training dataset.}
  \item{Yt}{response vector for test dataset.}
  \item{Xt}{ovariates matrix for test dataset.}
}

\author{Kun Chen, Wenxin Jiang, Martin A. Tanner.}

\examples{ 
    library(bvsgp)
    ###Simulate data from five dot model.###     
    simd <- fivedot_sim(K=50,n=30,nt=200)
    Y <- simd$Y
    X <- simd$X
    Yt <- simd$Yt
    Xt <- simd$Xt

    Beta <- gp_pmh(Y,X,N=500,burn=1500)

    ###Training error###
    fit<- X%*%Beta>0
    mean(apply((2*fit-1)*(2*Y-1)==-1,2,sum)/length(Y))

    ###Test error###
    pred <- Xt%*%Beta>0
    mean(apply((2*pred-1)*(2*Yt-1)==-1,2,sum)/length(Yt))

}
\keyword{datagen}% at least one, from doc/KEYWORDS
